/*
   версия 1.3
   лимитер полетного времени для планерных соревнований, включается в разрыв управления каналом газа
   house50 собака ngs.ru

   - При включении читает текущее значение высоты, принимает его за начальное
   - Читает текущее положение стика газа, принимает его за нулевое
   - Разрешение газа
   - Ждет добавления газа для перехода к след стадии (двойный бипы)
   - Идет отсчет 30 секунд - время разрешения работы стика газа, проверка уровня газа на 0, и в случае более 10 секунд, переход к след пункту (одинарные бипы)
   - Блокировка газа
   - Ожидание 10 секунд (сплошной бип)
   - Замер текущей высоты
   - Сообщение результата измерений высоты бипами (где длинный бип это 0, короткие бипы от 1 до 9)

*/






// отладочные дела, вывод данных в компорт
//#define MYDEBUG
//#define OLED_CONNECTED







#ifdef OLED_CONNECTED

#include <OLED_I2C.h>
OLED  myOLED(SDA, SCL, 8);
extern uint8_t SmallFont[];
extern uint8_t MediumNumbers[];
extern uint8_t BigNumbers[];

#endif






#define buzPin        10
#define servoOutPin   5

// на 2й и 3й ноге Ардуины Нано возможно внешнее прерывание
#define servoInPin    2

#define ledPin        13


// уровень газа означающий старт или стоп двигателя от 0 до 1000 (фактически миллисекунды ШИМ)
// throttleStartLevel > throttleStopLevel что бы исключить ложные срабатывания от дребезга в тч

#define throttleStartLevel  200
#define throttleStopLevel   100







//#include <Adafruit_BMP085.h>
#include <SFE_BMP180.h>



//float P0 = 0; // началь Altitude
//float P1 = 0; // конечн Altitude

SFE_BMP180 pressure;
double baseline, final_alt;


long time;
long time0;

#ifdef MYDEBUG
unsigned long time_allow_print;
#endif



/*
    замеры по сервотестеру показали:
    15.5 милисек низкий уровень, 1 милисек высокий уровень = МИНИМАЛЬНЫЙ ГАЗ
    14.5 милисек низкий уровень, 2 милисек высокий уровень = МАКСИМАЛЬНЫЙ ГАЗ
*/




int current_pulse = LOW;
int prev_pulse = LOW;

long pulse_up_time = 0;
//long pulse_down_time = 0;
long pulse_current_time = 0;



long start_pulse_up_time = 0;  // стартовое "нулевой газ" время горба ШИМ
long start_pulse_down_time = 0; // стартовое "нулевой газ" время ямы ШИМ



int stage = 0;  // уровень выполнения алгоритма







/*
   подключение датчика давления
   A4 = SDA
   A5 = SCL
   GND = GND
   RAW = VIN

*/



int i;




















int beepCount = 0; // количество бипов и пауз в стеке результата
char beepSeq[100]; // стек бипов и пауз для выдачи результата


void PushBeepDigit(int digit) // добавляет бипы одной цифры в стек
{
  int n;
  if (digit == 0) // ноль у нас обозначается длинным бипом
  {
    beepSeq[beepCount++] = LOW;
    beepSeq[beepCount++] = HIGH;
    beepSeq[beepCount++] = HIGH;
    beepSeq[beepCount++] = HIGH;
  } else
  {
    for (n = 0; n < digit; n++) // все остальные цифры короткими бипами
    {
      beepSeq[beepCount++] = LOW;
      beepSeq[beepCount++] = HIGH;
    }
  }
}













void FillBeeps(int number) // заполняет стек бипов всеми цифрами
{
  if ((number > 999) || (number < 0) ) number = 999; // чет пошло не так

  int n;
  for (n = 0; n < 30; n++) // заполним "паузу" между повторами цифирь
  {
    beepSeq[beepCount++] = LOW;
  }


  do
  {

    for (n = 0; n < 5; n++) // заполним "паузу" между самими цифрами
    {
      beepSeq[beepCount++] = LOW;
    }



    PushBeepDigit(number % 10); // добавляем одну цифру в стек бипов
    number = number / 10;
  } while (number); // как только НОЛЬ, значит стек заполнен, конец
}





volatile int allowPPMCopy = 0;
volatile int lastPPMlevel = 0;
volatile int curPPMlevel = 0;

volatile long in_pulse_up_time = 0; // длительность управляющего горба ШИМ
volatile long in_pulse_down_time = 0;  // длительность низкого уровня ШИМ
volatile long curPPMtime = 0;
volatile long lastPPMtime = 0;


void InPPMFrontChange() // прерывание по смене уровня ШИМ
{
  curPPMlevel = digitalRead(servoInPin);

  if (allowPPMCopy)
    digitalWrite(servoOutPin, curPPMlevel);


  curPPMtime = micros();

  if ( (curPPMlevel == HIGH) && (lastPPMlevel == LOW) ) // переход из низкого в высокий уровня ШИМ
  {
    in_pulse_down_time = curPPMtime - lastPPMtime; // получаем длительность ямы ШИМ

  } else  // переход из высокого в низкий уровня ШИМ
  {
    in_pulse_up_time = curPPMtime - lastPPMtime; // получаем длительность управляющего горба ШИМ
  }

  lastPPMtime = curPPMtime;
  lastPPMlevel = curPPMlevel;
}






void setup() {





  pinMode(buzPin, OUTPUT);
  pinMode(ledPin, OUTPUT);

  pinMode(servoOutPin, OUTPUT);

  pinMode(servoInPin, INPUT);
  attachInterrupt(0, InPPMFrontChange, CHANGE);

  pressure.begin();

#ifdef MYDEBUG
  Serial.begin(9600);
  Serial.print("start!\n\r");


  Serial.print("beepCount: ");
  Serial.print(beepCount);
  Serial.print("\n\r");

#endif

#ifdef OLED_CONNECTED
  myOLED.begin();
  //myOLED.invert(true);
  myOLED.setBrightness(10);
  myOLED.clrScr();

  myOLED.setFont(SmallFont);

  myOLED.print("DIFF:", LEFT, 0);
  myOLED.print("PRES:", LEFT, 24);
  myOLED.print("ALT:", LEFT, 48);

  myOLED.setFont(MediumNumbers);

#endif








}


long out_pulse_current_time;



void ThrottleDown()
{
  allowPPMCopy = 0;
  long m;

  m = micros();







  if (m - pulse_current_time > start_pulse_up_time) // делаем газ в ноль
  {
    digitalWrite(servoOutPin, LOW); // 1ms высокий уровень для нормального газа
  } else
  {
    digitalWrite(servoOutPin, HIGH); // 19ms низкий уровень для нормального газа
  }


  if (m - pulse_current_time > start_pulse_down_time + start_pulse_up_time) // конец цикла ППМ?
  {
    pulse_current_time = m;
  }


}


/*
  void CalculatePulse()   // считает время высокого и ниского уровня ШИМ и помещает их в глобальные переменые pulse_down_time и pulse_up_time
  {
  current_pulse = digitalRead(servoInPin);

  if ((prev_pulse == LOW) && (current_pulse == HIGH)) // уровень на входе с приемника сменился с низкого на высокий
  {
    pulse_down_time = micros() - pulse_current_time; // получаем длительность ямы ППМ
    pulse_current_time = micros();
  }



  if ((prev_pulse == HIGH) && (current_pulse == LOW)) // уровень на входе с приемника сменился с высокого на низкий
  {
    pulse_up_time = micros() - pulse_current_time; // получаем длительность горба ППМ (от 1000 микросек до 2000 микросек, мин и макс газа соответственно)
    pulse_current_time = micros();
  }
  prev_pulse = current_pulse;
  }


*/





void AltInitialization() // измеряет 10кратно начальное давление
{




  if (micros() - time0 > 100000) // 0.1 сек
  {
    baseline += getPressure();



    if (stage++ == 9) // 10 измерений
    {
      stage = 50;
      baseline = baseline / 10;

#ifdef OLED_CONNECTED
      myOLED.printNumF(baseline, 2, RIGHT, 20);
      myOLED.update();
#endif

#ifdef MYDEBUG

      Serial.print(baseline);
      Serial.print(" baseline presure\n\r");
#endif

      time0 = micros();

    }

  }
}









void TrottleLevelRead() // измеряет 10кратно начальное значение "нулевого" газа
{
  //  CalculatePulse();

  if (micros() - time0 > 100000) // 0.1 сек
  {

  //    in_pulse_down_time длительность ямы ШИМ
//    in_pulse_up_time длительность управляющего горба ШИМ

    start_pulse_up_time += in_pulse_up_time;   // десять измерений ШИМ (стик газа)
    start_pulse_down_time += in_pulse_down_time;


    if (stage++ == 59) // 10 измерений
    {
      stage = 100;

      start_pulse_up_time /= 10;   // десять измерений ШИМ (стик газа)
      start_pulse_down_time /= 10;


#ifdef MYDEBUG

      Serial.print(start_pulse_up_time);
      Serial.print(" ");
      Serial.print(start_pulse_down_time);
      Serial.print("  UP DOWN PWM\n\r");

#endif

      time0 = micros();

    }

  }
}










void AltFinal()
{
  double a, P;

  if (micros() - time0 > 100000) // 0.1 сек
  {

    P = getPressure();
    a = pressure.altitude(P, baseline);

    final_alt += a;

    if (stage++ == 409)
    {
      final_alt /= 10;

      FillBeeps(final_alt); // результирующая высота

#ifdef MYDEBUG
      Serial.print(final_alt);
      Serial.print(" final altitude\n\r");

      Serial.print("StackXXX: ");

      int n;

      for (n = 0; n < beepCount; n++)
      {
        Serial.print((int)beepSeq[n]);
      }
      Serial.print("\n\r");

#endif

#ifdef OLED_CONNECTED
      myOLED.printNumF(final_alt, 2, RIGHT, 44);
      myOLED.update();
#endif



      time0 = micros();


      stage = 500; // идем на уровень пищания результата

    }

  }

  ThrottleDown();

}













long throttleDownTimer; // таймер нулевого газа, считает время, когда во время полета газ сброшен и в случае превышения переходит к измерению высоты





void DoubleBeeps()
{
  long diff = micros() - time0;
  if (diff > 1000000) // 1 сек
  {
    time0 = micros();
  }

  if (  ((diff > 0) && (diff < 50000L)) || ((diff > 200000L) && (diff < 250000L))    )
  {
    digitalWrite(buzPin, HIGH);
    digitalWrite(ledPin, HIGH);
  } else
  {
    digitalWrite(buzPin, LOW);
    digitalWrite(ledPin, LOW);
  }

  //  CalculatePulse();

  /*
    ThrottleCopy();
  */

  allowPPMCopy = 1;

  //digitalWrite(servoOutPin, current_pulse);





  if (abs(in_pulse_up_time - start_pulse_up_time) > throttleStartLevel) // ручка газа отклонена более чем на throttleStartLevel? переходим к стадии полета
  {
#ifdef MYDEBUG
    Serial.print(pulse_up_time  - start_pulse_up_time );
    Serial.print(" ABS\n\r");
#endif
    stage = 200;

    throttleDownTimer = micros(); // сбрасываем таймер нулевого газа
  }


}















// стадия разрешения работы мотора 30 секунд
void SingleBeeps()
{
  long diff = micros() - time0;
  if (diff > 1000000) // 1 сек
  {
    time0 = micros();
    stage++;
  }

  if (  (diff > 0) && (diff < 300000L)    )
  {
    digitalWrite(buzPin, HIGH);
    digitalWrite(ledPin, HIGH);
  } else
  {
    digitalWrite(buzPin, LOW);
    digitalWrite(ledPin, LOW);
  }

  //CalculatePulse();

  /*

    ThrottleCopy();
  */
  allowPPMCopy = 1;


  //  digitalWrite(servoOutPin, current_pulse);


  if (stage == 230)
    stage = 300; // переходим на стадию блокировки двигателя и паузу измерения высоты

  if (  (abs(in_pulse_up_time - start_pulse_up_time) < throttleStopLevel))      // ручка газа отклонена МЕНЕЕ чем на throttleStopLevel ?
  {
    if (micros() - throttleDownTimer > 10000000L) // стик в нуле более 10 секунд?
    {
      stage = 400; // переходим на стадию блокировки двигателя и измерение высоты
    }

  } else
  {
    throttleDownTimer = micros(); // газ не в нуле, сбросим таймер нуля
  }


}






void WaitBeforeHeightMeasurement() // ждем 10 секунд после блокировки газа, для перехода к измерению давления
{
  long diff = micros() - time0;

  digitalWrite(buzPin, HIGH);
  digitalWrite(ledPin, HIGH);

  if (diff > 1000000) // 1 сек
  {
    time0 = micros();
    stage++;
  }

  ThrottleDown(); // газ в ноль удерживаем

  if (stage == 309)
  {
    digitalWrite(buzPin, LOW);
    digitalWrite(ledPin, LOW);
    stage = 400; // переходим к уровню измерения финального давления
  }
}

/*
  int currentBeepDigit = 350;
  void BeepDigit()
  {
    if (currentBeepDigit / 100
  }
  350
*/




int curBeep = 0;
void beepResult()
{
  long diff = micros() - time0;
  if (diff > 200000) // 0.2 сек
  {
    if ((curBeep--) == 0)
    {
      curBeep = beepCount;
    }

    digitalWrite(buzPin, beepSeq[curBeep]);
    digitalWrite(ledPin, beepSeq[curBeep]);
    time0 = micros();
  }

  ThrottleDown(); // газ в ноль удерживаем

}








double getPressure()
{
  char status;
  double T, P, p0, a;

  // You must first get a temperature measurement to perform a pressure reading.

  // Start a temperature measurement:
  // If request is successful, the number of ms to wait is returned.
  // If request is unsuccessful, 0 is returned.

  status = pressure.startTemperature();
  if (status != 0)
  {
    // Wait for the measurement to complete:

    delay(status);

    // Retrieve the completed temperature measurement:
    // Note that the measurement is stored in the variable T.
    // Use '&T' to provide the address of T to the function.
    // Function returns 1 if successful, 0 if failure.

    status = pressure.getTemperature(T);
    if (status != 0)
    {
      // Start a pressure measurement:
      // The parameter is the oversampling setting, from 0 to 3 (highest res, longest wait).
      // If request is successful, the number of ms to wait is returned.
      // If request is unsuccessful, 0 is returned.

      status = pressure.startPressure(3);
      if (status != 0)
      {
        // Wait for the measurement to complete:
        delay(status);

        // Retrieve the completed pressure measurement:
        // Note that the measurement is stored in the variable P.
        // Use '&P' to provide the address of P.
        // Note also that the function requires the previous temperature measurement (T).
        // (If temperature is stable, you can do one temperature measurement for a number of pressure measurements.)
        // Function returns 1 if successful, 0 if failure.

        status = pressure.getPressure(P, T);
        if (status != 0)
        {
          return (P);
        }
        else Serial.println("er ret pres\n");
      }
      else Serial.println("er st pres\n");
    }
    else Serial.println("er ret temp\n");
  }
  else Serial.println("er st temp\n");
}










void loop() {

  switch (stage)
  {
    // 10 измерений давления, для точности
    case 0:
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
      AltInitialization();
      break;


    // 10 измерений стика газа, для точности
    case 50:
    case 51:
    case 52:
    case 53:
    case 54:
    case 55:
    case 56:
    case 57:
    case 58:
    case 59:
      TrottleLevelRead();
      break;



    // ждем дачи газа более 20% для начала отсчета таймера работы двигателя
    case 100:
      DoubleBeeps();
      break;


    // таймер работы двигателя 30 сек
    case 200:
    case 201:
    case 202:
    case 203:
    case 204:
    case 205:
    case 206:
    case 207:
    case 208:
    case 209:
    case 210:
    case 211:
    case 212:
    case 213:
    case 214:
    case 215:
    case 216:
    case 217:
    case 218:
    case 219:
    case 220:
    case 221:
    case 222:
    case 223:
    case 224:
    case 225:
    case 226:
    case 227:
    case 228:
    case 229:
      SingleBeeps();
      break;



    case 300:
    case 301:
    case 302:
    case 303:
    case 304:
    case 305:
    case 306:
    case 307:
    case 308:
    case 309:
      WaitBeforeHeightMeasurement();
      break;


    // 10 измерений давления, для точности
    case 400:
    case 401:
    case 402:
    case 403:
    case 404:
    case 405:
    case 406:
    case 407:
    case 408:
    case 409:
      AltFinal();
      break;

    // отпискиваем метры высоты до бесконечности
    case 500:
      beepResult();
      break;




    default:
      // if nothing else matches, do the default
      // default is optional
      break;
  }








}
